# K-means
Travail réalisé par : Dario GENGA

## Structure du projet

- files_utils (.c/.h) : Fichiers contenant les méthodes de manipulation de fichiers.
- values_utils (.c/.h) : Fichiers contenant les méthodes de swap, génération aléatoires de nombres et de calcul de distances.
- kmeans (.c/.h) : Fichiers contenant les méthodes appliquant l'algorithme K-means.
- main.c : Fichier principal du projet servant à exécuter l'algorithme.
- main : Exécutable du projet.
- values_utils_tests.c : Fichier de test pour les méthodes de calcul de distances.

## Exécuter le programme

Compiler le programme avec la commande `make`.

Exécuter le fichier `main` de l'une des manières suivantes :

`./main` : Sans arguments, le programme vous demandera de les saisir.

`./main <path_to_custom_data_source>` : Passe en paramètre le chemin d'un fichier source définit comme dans  l'énoncé.
Crée le fichier `output.txt` qui contiendra le résultat du clustering.

`./main <path_to_custom_data_source> <output_path>` : Même chose qu'avant mais l'utilisateur définit lui-même le chemin de sortie.

`./main <number of dimensions> <number of cluster> <path_to_data_source> <output_path>` : Configure intégralement le clustering avec les arguments du programme. Attention, le fichier source ne doit pas contenir les lignes pour le nombre de clusters et le nombre de dimensions. Un fichier CSV (séparé par des virgules) est recommandé.

Quelques exemples d'exécution : 

`./main source_data.txt`

`./main 2 5 country.csv country_result.txt`

## Rappel de l'énoncé
Voir le fichier `enonce.pdf` pour la version au propre.

### But

Le but de ce travail est d'implémenter la méthode des k-moyennes
pour partitionner des données de façon non supervisée.

## Implémentation


### Format d'entrée

L'entrée sera un fichier spécifié en argument au programme.
Si aucun argument est fourni, le fichier d'entrée sera l'entrée standard (stdin).

Le fichier contiendra:

- sur la première ligne, le nombre de dimensions des points de l'ensemble
- sur la deuxième ligne, le nombre de clusters voulu
- un point par ligne, sous forme de N valeurs séparées par des virgules

```txt
<nombre de dimensions>
<nombre de clusters>
x1,x2,...,xn
y1,y2,...,yn
...
```


### Format de sortie

La sortie sera écrite dans un fichier spécifié en argument au programme.
Si aucun argument est fourni, le fichier de sortie sera la sortie standard (stdout).

Le fichier contiendra:

- sur la première ligne, le nombre de dimensions des points de l'ensemble
- sur la deuxième ligne, le nombre de clusters
- une étoile "*" signifie le début d'un cluster
- pour chaque étoile, une liste de points appartenant au même cluster,
  avec le même format que les points d'entrée

```txt
<nombre de dimensions>
<nombre de clusters>
*
x1,x2,...,xn
y1,y2,...,yn
...
*
z1,z2,...,zn
t1,t2,...,tn
...
```


### Interface utilisateur

Le programme fonctionnera entièrement en ligne de commande.
Les formats d'entrée et de sortie ont été choisis pour être
"pipe-friendly", ce qui donne un degré de flexibilité supplémentaire
à l'utilisateur.


### Déroulement de l'exécution du programme

- appel de la commande avec éventuellement le passage des
  fichiers d'entrée et de sortie en argument
- initialisation du nombre de dimensions de notre univers
- allocation de la mémoire pour les K groupes
- chargement des données d'entrée en mémoire
  (possibilité d'intervertir ce point avec le précédent)
- application de l'algorithme k-means
- écriture des résultats formatés dans le fichier de sortie
  (ou de la sortie standard, selon les arguments passés au programme)


### Fonctions à implémenter

Il faut absolument implémenter les fonctionnalités suivantes:

- gestion des arguments en ligne de commande
- lecture et écriture de fichiers formatés
- initialisation aléatoire et/ou intelligente des centroïdes
- au moins trois fonctions de calcul de similarité / "distance" entre deux points
- calcul du centre de gravité (_centroïde_) d'une partition
- les étapes de l'algorithme k-means


### Travail supplémentaire possible

#### Affichage en temps réel

Au lieu d'afficher uniquement le résultat final, il s'agirait
d'afficher chaque étape effectuée afin de pouvoir observer
l'algorithme en action.

#### Déterminer le nombre de clusters optimal

Faire des recherches sur les méthodes permettant d'optimiser
le nombre de clusters et les implémenter.
Si l'utilisation de cette fonctionnalité est le souhait de l'utilisateur,
un seul caractère non-numérique est utilisé en lieu du nombre
de clusters souhaité.
De légères modifications du format sont envisageables pour autant
qu'elles soient justifiées.

#### Interface graphique

On peut imaginer créer une interface graphique permettant de visualiser
le résultat dans une fenêtre graphique.
Pour cela, une option serait de mettre à profit la librairie gfx
utilisée pendant les labos de physique.


### Vérification du travail réalisé

Il serait bon de mettre en place des tests unitaires pour les
fonctions implémentées.

On implémente un algorithme non supervisé.
Il est donc difficile de vérifier si le résultat obtenu est "correct".
Néanmoins, un point placé dans le mauvais groupe est facile à identifier
par comparaison des distances du point aux différents centroïdes.


## Travail à rendre

- repo git contenant le code réalisé
- présentation du travail à l'aide d'un support (projection)
- démonstration du programme


## Références

https://www.editions-eni.fr/livre/le-machine-learning-avec-python-de-la-theorie-a-la-pratique-9782409031816/extrait-du-livre.pdf

https://towardsdatascience.com/k-means-clustering-algorithm-applications-evaluation-methods-and-drawbacks-aa03e644b48a

https://towardsmachinelearning.org/k-means/

https://vitalflux.com/elbow-method-silhouette-score-which-better/
