// Project : K-means
// Author : Dario GENGA

#ifndef _KMEANS_VALUES_UTILS_H
#define _KMEANS_VALUES_UTILS_H

#include <math.h>
#include <stdlib.h>


/// Get a random value in the min-max range.
/// \param min The minimal value (inclued).
/// \param max The maximal value (inclued).
/// \return A random int value.
int random_with_min_man_value(int min, int max);

/// Swap x and y values
/// \param x The first value
/// \param y The second value
void swap(int *x, int *y);

/// Compute the euclidean distance between two points of same dimensions.
/// \param p1 The values of the first point.
/// \param p2 The values of the second point.
/// \return The distance between the two points.
float compute_euclidean_distance(float* p1, float* p2, size_t dimensions);

/// Compute the manhattan distance between two points of same dimensions.
/// \param p1 The values of the first point.
/// \param p2 The values of the second point.
/// \return The distance between the two points.
float compute_manhattan_distance(float* p1, float* p2, size_t dimensions);

/// Compute the chebyshev distance between two points of same dimensions.
/// \param p1 The values of the first point.
/// \param p2 The values of the second point.
/// \return The distance between the two points.
float compute_chebyshev_distance(float* p1, float* p2, size_t dimensions);

#endif
